# -*- coding: utf-8 -*-

"""
    eve.io
    ~~~~~~

    This package implements the data layers supported by Eve.

    :copyright: (c) 2012 by Nicola Iarocci.
    :license: BSD, see LICENSE for more details.
"""

from base import DataLayer, ConnectionException
