# -*- coding: utf-8 -*-

"""
    eve.methods
    ~~~~~~~~~~~

    This package implements the HTTP methods supported by Eve.

    :copyright: (c) 2012 by Nicola Iarocci.
    :license: BSD, see LICENSE for more details.
"""

from get import get, getitem
from post import post
from patch import patch
from delete import delete
